<?php

namespace ICD\Hosting\Services;

use ICD\Hosting\ICD_Hosting;

/**
 * Class OrderHelper
 * @package ICD\Hosting\Services
 */
class OrderHelper {

	/**
	 * Stores order data
	 *
	 * @var array
	 */
	protected static $orderData;

	/**
	 * @var CatalogHelper
	 */
	protected static $catalogHelper;

	/**
	 * Handle catalog by location
	 */
	protected static $byLocation;

	/**
	 * @var
	 */
	public static $order;

	/**
	 * @param $orderData
	 */
	public static function init( $orderData ) {
		self::$orderData     = $orderData;
		self::$catalogHelper = new CatalogHelper( $orderData['catalog'], $orderData['products'] );
	}

	/**
	 * Filter catalog by location
	 *
	 * @param array $filter
	 *
	 * @return mixed
	 */
	public static function prepareCatalog( $filter = [] ) {
		if ( self::$byLocation ) {
			return self::$byLocation;
		}

		$catalog          = self::$catalogHelper->filterCatalog( $filter );
		self::$byLocation = self::$catalogHelper->catalogByLocation( $catalog );

		foreach ( self::$byLocation as $location => $items ) {
			foreach ( $items as $product => $details ) {
				if ( ! isset( $details['order'] ) ) {
					unset( self::$byLocation[ $location ][ $product ] );
				}
			}
		}

		return self::$byLocation;
	}

	/**
	 * Translate product resources
	 *
	 * @return array
	 */
	public static function translateResources() {
		$resources = array();
		foreach ( self::$orderData['products'] as $key => $val ) {
			if ( ! empty( $val['resources'] ) ) {
				foreach ( $val['resources'] as $rid => $rd ) {
					$val['resources'][ $rid ]['label'] = icd_hosting_tr( 'resources.' . $rd['name'] );
					$val['resources'][ $rid ]['label_unit'] = $rd['unit'] == 'COUNT' ? '' : icd_hosting_tr( 'units.' . $rd['unit'] );
				}
				$resources[ $key ] = $val['resources'];
			}
		}

		return icd_hosting_sanitize_all( $resources );
	}

	/**
	 * Translate countries
	 *
	 * @param null $countries
	 *
	 * @return |null
	 */
	public static function translateCountries( $countries = null ) {
		if ( is_null( $countries ) ) {
			$countries = self::$orderData['countries'];
		}

		$translated = ICD_Hosting::instance()->getApp()->trans->find_key( 'countries' );
		if ( ! $translated ) {
			return $countries;
		}

		foreach ( $translated as $k => $v ) {
			$translated[ $k ] = array_merge( $countries[ $k ], [ 'country' => $v ] );
		}

		return $translated;
	}

	/**
	 * Get offered top level domains
	 *
	 * @return array
	 */
	public static function offeredTLDs() {
		$tlds                = array();
		$catalog_by_location = self::prepareCatalog();
		foreach ( $catalog_by_location as $location => $items ) {
			foreach ( $items as $product => $details ) {
				if ( ! empty( $details['bonus:domain'] ) ) {
					$tlds = array_merge( $tlds, array_flip( array_keys( $details['bonus:domain'] ) ) );
				}
			}
		}

		return array_intersect_key( self::$orderData['tlds'], $tlds );
	}

	/**
	 * Get offered datacenters
	 * @return array
	 */
	public static function offeredDatacenters() {
		$result = array();
		foreach ( array_keys( self::prepareCatalog() ) as $dc ) {
			$result[ $dc ] = self::$orderData['datacenters'][ $dc ];
		}

		if ( isset( self::$orderData['datacenters'] ) ) {
			return array_intersect_key( $result, self::$orderData['datacenters'] );
		}

		return $result;
	}

	/**
	 * Format order
	 *
	 * @param $order
	 *
	 * @return mixed
	 */
	public static function formatOrder( $order ) {
		$order['total_paid'] = self::paymentsTotal( $order );
		$order['total_due']  = $order['total'] - $order['total_paid'];

		return $order;
	}

	/**
	 * Sum total payment for order
	 *
	 * @param $order
	 *
	 * @return float|int
	 */
	public static function paymentsTotal( $order ) {
		$sum = 0;
		if ( ! empty( $order['payments'] ) ) {
			foreach ( $order['payments'] as $payment ) {
				$sum += abs( $payment['total'] ) * ( $payment['payment_type'] == 'incoming' ? 1 : - 1 );
			}
		}

		return $sum;
	}

	/**
	 * Create order
	 *
	 * @param $input
	 *
	 * @return mixed
	 * @throws AppException
	 */
	public static function saveOrder( $input ) {
		if ( ! empty( $input['order_id'] ) ) {
			$order = self::getOrderDetails( $input['order_id'] );
			if ( $order['status'] == OrderStatusEnum::ORDER_NEW ) {
				return ICD_Hosting::instance()->getApp()->api->post( 'order-update', self::buildOrder( $input ) );
			}
		}

		return ICD_Hosting::instance()->getApp()->api->post( 'order-create', self::buildOrder( $input ) );
	}

	/**
	 * Build order data
	 *
	 * @param $input
	 *
	 * @return array
	 * @throws AppException
	 */
	public static function buildOrder( $input ) {
		if ( ! icd_hosting_valid_domain( $input['domain'] ) ) {
			icd_hosting_error( array( 'message' => 'domain_search.invalid', 'field' => 'domain', 'subtype' => 'invalid' ) );
		}

		// Get catalog & store data
		$order_data = ICD_Hosting::instance()->getApp()->api->get( 'order-data' );
		if ( empty( $order_data['data'] ) ) {
			icd_hosting_error( 'error.failed_to_retrieve_order_data' );
		}

		$order_data = $order_data['data'];

		self::init( $order_data );
		$catalog_by_location = self::prepareCatalog();

		if ( empty( $catalog_by_location[ $input['location'] ][ $input['plan'] ]['product_id'] ) ) {
			icd_hosting_error( 'error.product_not_found' );
		}

		// Get order & order items models
		self::$order = OrderDTO::getOrder();
		if ( ! empty( $input['order_id'] ) ) {
			self::$order['order_id'] = $input['order_id'];
		}

		$hosting_dto = OrderDTO::getHostingItem();

		if ( ! empty( $input['new_domain'] ) ) {
			$domain_dto = OrderDTO::getDomainItem();
			$sld_tld    = icd_hosting_sld_tld( $input['domain'] );

			if ( strpos( $input['plan'], 'multivps' ) !== false ) {
				if ( ! $sld_tld or empty( $catalog_by_location[ $input['location'] ][ $input['plan'] ]['extra:domain'][ $sld_tld['tld'] ]['order'][ $input['period'] ] ) ) {
					icd_hosting_error( 'error.tld_not_supported' );
				}
			} else if ( ! $sld_tld or empty( $catalog_by_location[ $input['location'] ][ $input['plan'] ]['bonus:domain'][ $sld_tld['tld'] ]['order'][ $input['period'] ] ) ) {
				icd_hosting_error( 'error.tld_not_supported' );
			}
		}

		if ( empty( $input['terms'] ) ) {
			AppException::error( array(
				'message' => 'required.you_must_agree_with_terms_of_use',
				'field'   => 'terms',
				'subtype' => 'required'
			) );
		}

		if ( ! empty( $input['new_domain'] ) and $order_data['tlds'][ $sld_tld['tld'] ]['icann'] and empty( $input['icann'] ) ) {
			AppException::error( array(
				'message' => 'required.you_must_agree_with_icann_terms',
				'field'   => 'icann',
				'subtype' => 'required'
			) );
		}

		if ( AppException::hasErrors() ) {
			throw new AppException;
		}

		// Fill order main/meta data
		self::$order['currency']       = $order_data['store']['currency'];
		self::$order['ip']             = $input['ip'];
		self::$order['referer']        = $input['referer'];
		self::$order['app']            = 'WORDPRESS';
		self::$order['payment_method'] = isset( $input['payment_method'] ) ?
			ICD_Hosting::instance()->getApp()->payment->get( $input['payment_method'] )->paymentMethod( $input['payment_method'] ) : 'Pending';

		self::$order['user_agent']          = isset( $_SERVER['HTTP_USER_AGENT'] ) ? $_SERVER['HTTP_USER_AGENT'] : '';
		self::$order['user_browser_locale'] = isset( $_SERVER['HTTP_ACCEPT_LANGUAGE'] ) ? $_SERVER['HTTP_ACCEPT_LANGUAGE'] : '';

		// Fill contact data where applicable
		foreach ( $input['contact'] as $key => $val ) {
			if ( array_key_exists( $key, self::$order ) ) {
				self::$order[ $key ] = $val;
			}
		}

		// Fill hosting product props
		$plan_item                       = $catalog_by_location[ $input['location'] ][ $input['plan'] ];
		$hosting_dto['hosting.hostname'] = icd_hosting_valid_domain( $input['domain'] );
		$hosting_dto['action']           = 'order';
		$hosting_dto['period']           = $input['period'];
		$hosting_dto['catalog_id']       = $plan_item['catalog_id'];
		$hosting_dto['type']             = $hosting_dto['product_type'] = $plan_item['product_type'];
		$hosting_dto['product']          = $hosting_dto['item'] = $plan_item['product'];
		$hosting_dto['name']             = $plan_item['name'] . ', ' . icd_hosting_tr( 'locations.' . $input['location'] ) . ': ' . $hosting_dto['hosting.hostname'];

		$hosting_dto['price'] = $plan_item['order'][ $input['period'] ]['price'];

		//create new or update existing hosting item
		$hosting_item_id = empty( $input['hosting_item_id'] ) ? 0 : $input['hosting_item_id'];
		if ( $hosting_item_id !== 0 ) {
			$hosting_dto['item_id'] = $hosting_item_id;
		}

		self::$order['items'][ $hosting_item_id ] = $hosting_dto;

		if ( isset( $domain_dto ) ) {
			if ( strpos( $input['plan'], 'multivps' ) !== false ) {
				$domain_item = $catalog_by_location[ $input['location'] ][ $input['plan'] ]['extra:domain'][ $sld_tld['tld'] ];
			} else {
				$domain_item = $catalog_by_location[ $input['location'] ][ $input['plan'] ]['bonus:domain'][ $sld_tld['tld'] ];
			}
			$domain_dto['domain.tld'] = $sld_tld['tld'];
			$domain_dto['domain.sld'] = $sld_tld['sld'];
			$domain_dto['action']     = 'order';
			$domain_dto['period']     = $input['period'];
			if ( $plan_item['periodicity'] == 'MO' ) {
				$domain_dto['period'] = $domain_dto['period'] <= 12 ? 1 : intdiv( $domain_dto['period'], 12 );
			}
			$domain_dto['catalog_id'] = $domain_item['catalog_id'];
			$domain_dto['type']       = $domain_dto['product_type'] = $domain_item['product_type'];
			$domain_dto['product']    = $domain_dto['item'] = $domain_item['product'];
			$domain_dto['name']       = $domain_item['name'] .
			                            ': ' . $sld_tld['sld'] . '.' . $sld_tld['tld'];

			$domain_dto['price'] = $domain_item['order'][ $input['period'] ]['price'];

			if ( ! empty( $input['domain_extra_attributes'] ) ) {
				$domain_dto['domain.extra_attributes'] = $input['domain_extra_attributes'];
			}

			//create new or update existing domain item
			$domain_item_id = empty( $input['domain_item_id'] ) ? 1 : $input['domain_item_id'];
			if ( $domain_item_id !== 1 ) {
				$domain_dto['item_id'] = $domain_item_id;
			}

			$domain_dto['parent']                    = $hosting_item_id;
			self::$order['items'][ $domain_item_id ] = $domain_dto;
		}

		return icd_hosting_sanitize_all( self::$order );
	}

	/**
	 * Get order details
	 *
	 * @param $order_id
	 *
	 * @return mixed
	 * @throws AppException
	 */
	public static function getOrderDetails( $order_id ) {
		if ( ! preg_match( '/[A-Z0-9]{16}/i', $order_id ) ) {
			icd_hosting_error( 'invalid.order_id' );
		}

		$result = ICD_Hosting::instance()->getApp()->api->get( 'order-details', array( 'order_id' => $order_id ) );
		if ( empty( $result['data']['order'] ) ) {
			icd_hosting_error( 'error.failed_to_retrieve_order_data' );
		}

		return icd_hosting_sanitize_all( $result['data']['order'] );
	}

	/**
	 * Get payment request details
	 *
	 * @param $request_id
	 *
	 * @return mixed
	 * @throws AppException
	 */
	public static function getPaymentRequestDetails( $request_id ) {
		$result = ICD_Hosting::instance()->getApp()->api->get( 'payment-request-details', array( 'payment_request_id' => $request_id ) );
		if ( empty( $result['data']['payment_request'] ) ) {
			icd_hosting_error( 'error.failed_to_retrieve_order_data' );
		}

		return  $result['data']['payment_request'];
	}

	/**
	 * Get order request details
	 *
	 * @param $request_id
	 *
	 * @return mixed
	 * @throws AppException
	 */
	public static function getRequestDetails( $request_id ) {
		if ( ! preg_match( '/[0-9]{1,16}/i', $request_id ) ) {
			icd_hosting_error( 'invalid.request_id:' . $request_id );
		}

		$result = ICD_Hosting::instance()->getApp()->api->get( 'order-request-details', [ 'id' => $request_id ] );
		if ( empty( $result['data']['order'] ) ) {
			icd_hosting_error( 'error.failed_to_retrieve_order_request' );
		}

		return $result['data']['order'];
	}
}
