<?php

namespace ICD\Hosting\Services\Processors;

/**
 * Class Payment2CheckOut
 * @package ICD\Hosting\Services\Processors
 */
class Payment2CheckOut extends PaymentProcessor {

	/**
	 * @var
	 */
	protected $merchantId;
	/**
	 * @var
	 */
	protected $secret;

	/**
	 * Payment2CheckOut constructor.
	 *
	 * Initialize 2Checkout ID and Secret and sandbox gateway
	 *
	 * @param array $config
	 */
	public function __construct( $config = array() ) {
		parent::__construct( $config, $config['test'] ? 'https://sandbox.2checkout.com/checkout/purchase' : 'https://www.2checkout.com/checkout/purchase' );

		$this->merchantId = $config['options']['id'];
		$this->secret     = $config['options']['secret'];
	}

	/**
	 * Payment method specific attributes
	 *
	 * @param $order_id
	 * @param $amount
	 * @param $currency
	 * @param array $order
	 * @param int $discount
	 *
	 * @return array
	 */
	public function paymentData( $order_id, $amount, $currency, $order = array(), $discount = 0 ) {
		$i           = 0;
		$item_params = array();
		$cart_total  = 0;
		if ( $discount ) {
			$item_params["li_0_type"] = 'order';
			$item_params["li_0_name"] = icd_hosting_tr( 'hosting_order_due', array( 'order_id' => $order_id ) );
			//$item_params["li_0_quantity"] = 1;
			$item_params["li_0_price"] = $amount - $discount;
		} else {
			foreach ( $order['items'] as $item ) {
				$item_params["li_{$i}_type"] = 'product';
				$item_params["li_{$i}_name"] = icd_hosting_format_item_label( $item );
				//$item_params["li_{$i}_quantity"] = isset($item['quantity']) ? $item['quantity'] : 1;
				$item_params["li_{$i}_price"] = $this->exchangeAmount( $item['price'] );
				$cart_total                   += $item_params["li_{$i}_price"];
				$i ++;
			}
			if ( $cart_total > $amount ) {
				$item_params              = array();
				$item_params["li_0_type"] = 'order';
				$item_params["li_0_name"] = icd_hosting_tr( 'hosting_order_due', array( 'order_id' => $order_id ) );
				//$item_params["li_0_quantity"] = 1;
				$item_params["li_0_price"] = $amount;
			}
		}

		return array(
			'url'    => $this->url,
			'type'   => 'online',
			'params' => array(
				            'sid'                => $this->merchantId,
				            'mode'               => '2CO',
				            'total'              => bcsub( (string) $amount, (string) $discount, 2 ),
				            'currency_code'      => $currency,
				            //'merchant_order_id' => $order_id,
				            'merchant_order_id'  => str_pad( $this->request_id, 11, '0', STR_PAD_LEFT ),
				            'return_url'         => $this->postbackUrl(),
				            'x_receipt_link_url' => $this->postbackUrl(),
				            'card_holder_name'   => $order['firstname'] . ' ' . $order['lastname'],
				            'street_address'     => $order['address'],
				            'city'               => $order['city'],
				            'state'              => $order['state'],
				            'zip'                => $order['zip'],
				            'country'            => $order['country'],
				            'email'              => $order['email'],
				            'phone'              => '(' . $order['phone_country_code'] . ')' . $order['phone'],
			            ) + $item_params,
		);
	}

	/**
	 * Verifies that the payment is valid
	 *
	 * @param $input
	 *
	 * @return array|bool
	 */
	public function validPayment( $input ) {
		if ( isset( $input['key'] ) and isset( $input['order_number'] ) and isset( $input['total'] ) ) {
			$hash = strtoupper( md5( $this->secret . $this->merchantId . $input['order_number'] . $input['total'] ) );

			if ( $hash == $input['key'] ) {
				return array(
					'payment_processor_id' => $this->config['processor_id'],
					'order_id'             => null,
					'payment_request_id'   => $input['merchant_order_id'],
					'payment_type'         => 'incoming',
					'payment_method'       => $this->paymentMethod(),
					'transaction_id'       => $input['order_number'],
					'payment_date'         => date( 'Y-m-d H:i:s T' ),
					'merchant_id'          => $this->merchantId,
					'total'                => $input['total'],
					'ipn'                  => $input,
				);
			}
		}

		if ( isset( $input['md5_hash'] ) and isset( $input['sale_id'] ) and isset( $input['invoice_id'] ) ) {
			$hash = strtoupper( md5( $input['sale_id'] . $this->merchantId . $input['invoice_id'] . $this->secret ) );

			if ( $hash == $input['md5_hash'] ) {
				return array(
					'payment_processor_id' => $this->config['processor_id'],
					'order_id'             => null,
					'payment_request_id'   => $input['vendor_order_id'],
					'payment_type'         => $this->paymentType( $input ),
					'payment_method'       => $this->paymentMethod(),
					'transaction_id'       => $input['sale_id'],
					'payment_date'         => $input['timestamp'] . ' EST',
					'merchant_id'          => $this->merchantId,
					'total'                => $this->paymentTotal( $input ),
					'ipn'                  => $input,
				);
			}
		}
	}

	/**
	 * Get payment type
	 *
	 * @param $ipn
	 *
	 * @return string
	 */
	protected function paymentType( $ipn ) {
		if ( in_array( $ipn['message_type'], array( 'ORDER_CREATED' ) ) ) {
			return 'incoming';
		} else if ( in_array( $ipn['message_type'], array( 'REFUND_ISSUED' ) ) ) {
			return 'refund';
		}

		return 'error';
	}

	/**
	 * Total payment amount
	 *
	 * @param $ipn
	 *
	 * @return int
	 */
	protected function paymentTotal( $ipn ) {
		$payment_type = $this->paymentType( $ipn );
		if ( $payment_type == 'incoming' ) {
			return $ipn['invoice_list_amount'];
		}

		if ( $payment_type == 'refund' ) {
			$total = 0;
			for ( $i = 1; $i <= $ipn['item_count']; $i ++ ) {
				if ( $ipn["item_type_$i"] == 'refund' ) {
					$total -= $ipn["item_list_amount_$i"];
				}
			}

			return $total;
		}
	}
}
