<?php

namespace ICD\Hosting\Services\Processors;

/**
 * Class PaymentPayPal
 *
 * @package ICD\Hosting\Services\Processors
 */
class PaymentPayPal extends PaymentProcessor {

	/**
	 * @var
	 */
	protected $merchantId;

	/**
	 * PaymentPayPal constructor.
	 *
	 * Initialize PayPal merchant ID and sandbox gateway
	 *
	 * @param array $config
	 */
	public function __construct( $config = array() ) {
		parent::__construct( $config, $config['test'] ? 'https://www.sandbox.paypal.com/cgi-bin/webscr' : 'https://www.paypal.com/cgi-bin/webscr' );

		$this->merchantId = $config['options']['id'];
	}

	/**
	 * Payment method specific attributes
	 *
	 * @param $order_id
	 * @param $amount
	 * @param $currency
	 * @param array $order
	 * @param int $discount
	 *
	 * @return array
	 */
	public function paymentData( $order_id, $amount, $currency, $order = array(), $discount = 0 ) {
		$i           = 0;
		$item_params = array();
		$cart_total  = 0;
		foreach ( $order['items'] as $item ) {
			$i ++;
			$item_params["item_name_$i"] = icd_hosting_format_item_label( $item );
			//$item_params["quantity_$i"] = isset($item['quantity']) ? $item['quantity'] : 1;
			$item_params["amount_$i"] = $this->exchangeAmount( $item['price'] );
			$cart_total               += $item_params["amount_$i"];
		}
		if ( $cart_total > $amount ) {
			$discount = $cart_total - $amount;
		}

		return array(
			'url'    => $this->url,
			'type'   => 'online',
			'params' => array(
				            'cmd'                  => '_cart',
				            'upload'               => 1,
				            'business'             => $this->merchantId,
				            'amount'               => $amount,
				            'currency_code'        => $currency,
				            'discount_amount_cart' => $discount,
				            'invoice'              => str_pad( $this->request_id, 11, '0', STR_PAD_LEFT ),
				            'no_note'              => 1,
				            'no_shipping'          => 1,
				            'return'               => $this->returnUrl( array( 'order_id' => $order_id ) ),
				            'cancel_return'        => $this->cancelUrl( array( 'order_id' => $order_id ) ),
				            'notify_url'           => $this->postbackUrl(),
				            'charset'              => 'utf-8'
			            ) + $item_params
		);
	}

	/**
	 * Detect a valid payment
	 *
	 * @param $input
	 *
	 * @return array|bool
	 */
	public function detectPayment( $input ) {
		if ( $this->validPayment( $input ) ) {
			return array(
				'payment_processor_id' => $this->config['processor_id'],
				'order_id'             => null,
				'payment_request_id'   => intval( $input['invoice'] ),
				'payment_type'         => $this->paymentType( $input ),
				'payment_method'       => $this->paymentMethod(),
				'transaction_id'       => $input['txn_id'],
				'payment_date'         => date( 'Y-m-d H:i:s T', strtotime( $input['payment_date'] ) ),
				'merchant_id'          => $input['receiver_email'],
				'total'                => $this->paymentTotal( $input ),
				'ipn'                  => $input,
			);
		}
	}

	/**
	 * Verifies that the payment is valid
	 *
	 * @param $input
	 *
	 * @return bool
	 */
	public function validPayment( $input ) {
		// STEP 1: read POST data
		// Reading POSTed data directly from $_POST causes serialization issues with array data in the POST.
		// Instead, read raw POST data from the input stream.
		$raw_post_data  = file_get_contents( 'php://input' );
		$raw_post_array = explode( '&', $raw_post_data );
		$myPost         = array();
		foreach ( $raw_post_array as $keyval ) {
			$keyval = explode( '=', $keyval );
			if ( count( $keyval ) == 2 ) {
				$myPost[ $keyval[0] ] = urldecode( $keyval[1] );
			}
		}

		if ( ! $myPost ) {
			return false;
		}

		// read the IPN message sent from PayPal and prepend 'cmd=_notify-validate'
		$req = 'cmd=_notify-validate';
		if ( function_exists( 'get_magic_quotes_gpc' ) ) {
			$get_magic_quotes_exists = true;
		}

		foreach ( $myPost as $key => $value ) {
			if ( $get_magic_quotes_exists && get_magic_quotes_gpc() ) {
				$value = urlencode( stripslashes( $value ) );
			} else {
				$value = urlencode( $value );
			}

			$req .= "&$key=$value";
		}

		// Step 2: POST IPN data back to PayPal to validate
		$ch = curl_init( $this->url );
		curl_setopt( $ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1 );
		curl_setopt( $ch, CURLOPT_POST, 1 );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
		curl_setopt( $ch, CURLOPT_POSTFIELDS, $req );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, 1 );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, 2 );
		curl_setopt( $ch, CURLOPT_FORBID_REUSE, 1 );
		curl_setopt( $ch, CURLOPT_HTTPHEADER, array( 'Connection: Close', 'User-Agent: IPN Verification' ) );

		$res = curl_exec( $ch );
		curl_close( $ch );

		return $res === 'VERIFIED' ? true : false;
	}

	/**
	 * Get payment type based on IPN payment status
	 *
	 * @param $ipn
	 *
	 * @return string
	 */
	protected function paymentType( $ipn ) {
		//Canceled_Reversal, Completed, Created, Processed
		$type = 'incoming';

		if ( in_array( $ipn['payment_status'], array(
			'Denied',
			'Expired',
			'Failed',
			'Refunded',
			'Reversed',
			'Voided'
		) ) ) {
			$type = in_array( $ipn['reason_code'], array(
				'chargeback',
				'guarantee',
				'buyer-complaint'
			) ) ? 'chargeback' : 'refund';
		}

		return $type;
	}

	/**
	 * Total payment amount
	 *
	 * @param $ipn
	 *
	 * @return mixed
	 */
	protected function paymentTotal( $ipn ) {
		return $ipn['mc_gross'];
	}
}
