<?php

namespace ICD\Hosting\Services\Processors;

use ICD\Hosting\ICD_Hosting;
use ICD\Hosting\Services\AppException;

/**
 * Class PaymentStripe
 *
 * @package ICD\Hosting\Services\Processors
 */
class PaymentStripe extends PaymentProcessor {
	/**
	 * PaymentStripe constructor.
	 *
	 * @param $config
	 */
	public function __construct( $config ) {
		parent::__construct( $config );
	}

	/**
	 * Make payment using stripe library
	 *
	 * @param $order_id
	 * @param $amount
	 * @param $currency
	 * @param array $order
	 * @param array $input
	 *
	 * @return array
	 * @throws AppException
	 * @throws ProcessorException
	 */
	public function makePayment( $order_id, $amount, $currency, $order = array(), $input = array() ) {
		try {
			$secret_key = $this->config['options']['secret_key'];
			$params     = array(
				'amount'        => bcmul( (string) $amount, '100' ),
				'currency'      => $currency,
				'source'        => ! empty( $input['stripeToken'] ) ? $input['stripeToken'] : '',
				'description'   => icd_hosting_tr( 'hosting_order_due', array( 'order_id' => $order_id ) ),
				'receipt_email' => $order['email'],
			);

			ICD_Hosting::instance()->load_stripe_lib();

			\Stripe\Stripe::setApiKey( $secret_key );

			$charge = \Stripe\Charge::create( $params );

			$last_response = $charge->getLastResponse()->json; // <-- The name is misleading, this is actually an array
			unset( $last_response['source'] );

			$result = array(
				'payment_processor_id' => $this->config['processor_id'],
				'order_id'             => $order_id,
				'payment_request_id'   => null,
				'payment_method'       => $this->paymentMethod(),
				'transaction_id'       => $charge->id,
				'payment_date'         => gmdate( DATE_ATOM, $charge->created ),
				// <-- Stripe uses Unix timestamps, but 'payment-add' action expects string representation
				'merchant_id'          => $this->config['options']['id'],
				'ipn'                  => $last_response
				// <-- This is not a real IPN but the response from the charge action
			);

			return $result;
		} catch ( \Stripe\Error\Card $e ) {
			throw new AppException( $this->buildErrKey( $e ) );
		} catch ( \Stripe\Error\RateLimit $e ) {
			$this->handleException( $e );
		} catch ( \Stripe\Error\InvalidRequest $e ) {
			$this->handleException( $e );
		} catch ( \Stripe\Error\Authentication $e ) {
			$this->handleException( $e );
		} catch ( \Stripe\Error\ApiConnection $e ) {
			$this->handleException( $e );
		}
	}

	/**
	 * Build error key
	 *
	 * @param $e
	 *
	 * @return string
	 */
	private function buildErrKey( $e ) {
		if ( ! $e instanceof \Stripe\Error\Base ) {
			return 'error.generic';
		}

		$body = $e->getJsonBody();
		$err  = $body['error'];

		return "stripe.{$err['type']}.{$err['code']}";
	}

	/**
	 * Handle errors
	 *
	 * @param $e
	 *
	 * @throws ProcessorException
	 */
	private function handleException( $e ) {
		$msg = get_class( $e ) . "\n\n" . $e->getMessage();

		throw new ProcessorException( $msg, preg_replace( '/^Payment/', '', get_class( $this ) ) );
	}
}
